import os

import httpx

import gh_util
from gh_util.logging import get_logger

logger = get_logger(__name__)


class GHClient(httpx.AsyncClient):
    """A wrapper around httpx.AsyncClient that adds GitHub authentication."""

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        if gh_util.settings.token:
            logger.debug_kv(
                "AUTH",
                "Using GitHub token set in environment via `GH_UTIL_TOKEN`",
                "blue",
            )
            token = gh_util.settings.token.get_secret_value()
        elif token := os.getenv("GITHUB_TOKEN"):
            logger.debug_kv(
                "AUTH",
                "Using `GITHUB_TOKEN` set in environment",
                "blue",
            )
        else:
            logger.warning_kv(
                "AUTH",
                (
                    "`GH_UTIL_TOKEN` not set in env vars or `.env` - watch out for rate limits and 401s!"
                    " see https://docs.github.com/en/github/authenticating-to-github/creating-a-personal-access-token"
                ),
                "red",
            )
        self.headers.update(
            {
                "Accept": "application/vnd.github.v3+json",
                "Authorization": f"token {token}",
            }
        )

    async def request(self, method, url, *args, **kwargs) -> httpx.Response:
        """Allow passing in a relative URL."""
        url = str(url)
        if url.startswith("/"):
            url = f"{gh_util.settings.base_url}{url}"

        response = await super().request(method, url, *args, **kwargs)
        response.raise_for_status()
        return response

    async def get(self, url, *args, **kwargs) -> httpx.Response:
        return await self.request("GET", url, *args, **kwargs)

    async def post(self, url, *args, **kwargs) -> httpx.Response:
        return await self.request("POST", url, *args, **kwargs)
